/** \file DataHandler.h
    \brief Declaration of DataHandler class.
    \author James Peachey

    The Profit software package was developed by Dr. Randall Smith and
    Mr. James Peachey at the High Energy Astrophysics Science Archive
    Research Center (HEASARC) at the Goddard Space Flight Center (GSFC)
    of the National Aeronautics and Space Administration (NASA).
*/
#ifndef profit_DataHandler_h
#define profit_DataHandler_h

#include "profit/Datum.h"
#include "profit/DataView.h"

#include <set>
#include <string>

namespace profit {
  typedef std::string Message;

  /** \class DataHandler
      \brief Class encapsulating an event handler, which responds to messages from the graphical
      interface by updating data and in turn any views of the data.
  */
  class DataHandler {
    public:
      typedef std::set<Datum *> data_cont_type;
      typedef DataView::const_data_cont_type const_data_cont_type;
      typedef Datum::view_cont_type view_cont_type;

      virtual ~DataHandler();

      /** \brief Perform specific handling of a message sent by the graphical interface.
          \param message The message to handle.
          \param datum The Datum object associated with the message.
          \param affected_data Output data container which holds pointers to affected data objects.
      */
      virtual void handle(const Message & message, const Datum & datum, const_data_cont_type & affected_data) = 0;

      /** \brief Select the given data object from a container of other data objects.
          \param datum The data object being selected.
          \param deselect Container of data objects to de-select.
      */
      void select(Datum & datum, data_cont_type & deselect);

      /** \brief Handle a message sent by the graphical interface, according to a standard protocol.

          The standard protocol is to perform setup, call the handle method for specific handling, then
          update all views affected by the changes to the data, and call whatever clean-up is necessary.
          \param message The message to handle.
          \param datum The Datum object associated with the message.
      */
      void handleMessage(const Message & message, const Datum & datum);
  };

}

#endif
